@@ -22,15 +22,19 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 # POSSIBILITY OF SUCH DAMAGE.
 package SQL::Functional;
-$SQL::Functional::VERSION = '0.1';
-use v5.14;
+$SQL::Functional::VERSION = '0.2';
+use strict;
 use warnings;
 use SQL::Functional::AndClause;
 use SQL::Functional::DeleteClause;
+use SQL::Functional::DistinctClause;
 use SQL::Functional::FromClause;
+use SQL::Functional::FunctionClause;
 use SQL::Functional::GroupByClause;
-use SQL::Functional::InnerJoinClause;
 use SQL::Functional::InsertClause;
+use SQL::Functional::JoinClause;
+use SQL::Functional::LimitClause;
+use SQL::Functional::LiteralClause;
 use SQL::Functional::MatchClause;
 use SQL::Functional::NullClause;
 use SQL::Functional::OrClause;
@@ -39,6 +43,7 @@ use SQL::Functional::PlaceholderClause;
 use SQL::Functional::SelectClause;
 use SQL::Functional::SetClause;
 use SQL::Functional::TableClause;
+use SQL::Functional::TruncateClause;
 use SQL::Functional::UpdateClause;
 use SQL::Functional::ValuesClause;
 use SQL::Functional::VerbatimClause;
@@ -61,6 +66,14 @@ our @EXPORT_OK = qw{
     ORDER_BY
     DESC
     INNER_JOIN
+    JOIN
+    LEFT_JOIN
+    LEFT_OUTER_JOIN
+    RIGHT_JOIN
+    RIGHT_OUTER_JOIN
+    FULL_JOIN
+    FULL_OUTER_JOIN
+    OUTER_JOIN
     SUBSELECT
     AND
     OR
@@ -74,6 +87,11 @@ our @EXPORT_OK = qw{
     IS_NULL
     IS_NOT_NULL
     GROUP_BY
+    LIMIT
+    TRUNCATE
+    DISTINCT
+    func
+    literal
 };
 our @EXPORT = @EXPORT_OK;
 
@@ -84,6 +102,8 @@ sub SELECT ($$@)
 {
     my ($fields, @clauses) = @_;
     my @fields;
+    my $is_distinct = 0;
+
     if( ref $fields eq 'ARRAY' ) {
         @fields = map {
             (ref($_) && $_->isa( 'SQL::Functional::FieldClause' ))
@@ -96,6 +116,10 @@ sub SELECT ($$@)
     elsif( $fields->isa( 'SQL::Functional::FieldClause' ) ) {
         @fields = ($fields);
     }
+    elsif( $fields->isa( 'SQL::Functional::DistinctClause' ) ) {
+        @fields = ($fields->fields);
+        $is_distinct = 1;
+    }
     else {
         @fields = ( SQL::Functional::FieldClause->new({
             name => $fields,
@@ -105,6 +129,7 @@ sub SELECT ($$@)
     my $clause = SQL::Functional::SelectClause->new(
         fields => \@fields,
         clauses => \@clauses,
+        is_distinct => $is_distinct,
     );
     return ($clause->to_string, $clause->get_params);
 }
@@ -137,7 +162,15 @@ sub FROM (@)
 {
     my (@tables) = @_;
     my @table_objs = map {
-        ref $_ ? $_ : table $_
+        my $result;
+        if( ref $_ ) {
+            $result = $_;
+        }
+        else {
+            $result = table $_;
+        }
+
+        $result;
     } @tables;
 
     my $clause = SQL::Functional::FromClause->new({
@@ -228,14 +261,57 @@ sub DESC($)
 sub INNER_JOIN($$$)
 {
     my ($table, $field1, $field2) = @_;
-    my $clause = SQL::Functional::InnerJoinClause->new(
+    my $clause = SQL::Functional::JoinClause->new(
         table => $table,
         field1 => $field1,
         field2 => $field2,
+        type => 'inner',
     );
     return $clause;
 }
 
+sub LEFT_JOIN($$$)
+{
+    my ($table, $field1, $field2) = @_;
+    my $clause = SQL::Functional::JoinClause->new(
+        table => $table,
+        field1 => $field1,
+        field2 => $field2,
+        type => 'left',
+    );
+    return $clause;
+}
+
+sub RIGHT_JOIN($$$)
+{
+    my ($table, $field1, $field2) = @_;
+    my $clause = SQL::Functional::JoinClause->new(
+        table => $table,
+        field1 => $field1,
+        field2 => $field2,
+        type => 'right',
+    );
+    return $clause;
+}
+
+sub FULL_JOIN($$$)
+{
+    my ($table, $field1, $field2) = @_;
+    my $clause = SQL::Functional::JoinClause->new(
+        table => $table,
+        field1 => $field1,
+        field2 => $field2,
+        type => 'full',
+    );
+    return $clause;
+}
+
+*JOIN = \&INNER_JOIN;
+*LEFT_OUTER_JOIN = \&LEFT_JOIN;
+*RIGHT_OUTER_JOIN = \&RIGHT_JOIN;
+*FULL_OUTER_JOIN = \&FULL_JOIN;
+*OUTER_JOIN = \&FULL_JOIN;
+
 sub SUBSELECT($$@)
 {
     my ($fields, @clauses) = @_;
@@ -388,6 +464,74 @@ sub GROUP_BY ($)
     return $clause;
 }
 
+sub LIMIT ($)
+{
+    my ($num) = @_;
+    my $limit = SQL::Functional::LimitClause->new({
+        num => $num,
+    });
+    return $limit;
+}
+
+sub TRUNCATE($)
+{
+    my ($table) = @_;
+    my $trunc = SQL::Functional::TruncateClause->new({
+        table => $table,
+    });
+    return ($trunc->to_string, $trunc->get_params);
+}
+
+sub DISTINCT ($)
+{
+    my ($fields) = @_;
+    my @fields;
+
+    if( ref $fields eq 'ARRAY' ) {
+        @fields = map {
+            (ref($_) && $_->isa( 'SQL::Functional::FieldClause' ))
+                ? $_
+                : SQL::Functional::FieldClause->new({
+                    name => $_,
+                });
+        } @$fields;
+    }
+    elsif( $fields->isa( 'SQL::Functional::FieldClause' ) ) {
+        @fields = ($fields);
+    }
+    else {
+        @fields = ( SQL::Functional::FieldClause->new({
+            name => $fields,
+        }) );
+    }
+
+    my $clause = SQL::Functional::DistinctClause->new({
+        fields => \@fields,
+    });
+    return $clause;
+}
+
+sub func ($;@)
+{
+    my ($func_name, @args) = @_;
+
+    my $clause = SQL::Functional::FunctionClause->new({
+        name => $func_name,
+        args => \@args,
+    });
+
+    return $clause;
+}
+
+sub literal ($)
+{
+    my ($str) = @_;
+    my $clause = SQL::Functional::LiteralClause->new({
+        literal => $str,
+    });
+    return $clause;
+}
+
 
 1;
 __END__
@@ -574,11 +718,57 @@ Used with C<ORDER_BY> to set a field to sort in decending order.
 
   INNER_JOIN( $table, $field1, $field2 );
 
-Creates a L<SQL::Functional::InnerJoinClause> and returns it. The first 
-argument, (C<$table>), is an C<SQL::Functional::TableClause> object, which 
-is the table being joined. The second argument is the field on the main 
-table that will be checked. The third argument is the field on the joined 
-table.
+Creates a L<SQL::Functional::JoinClause> with C<join_type = 'inner'> and 
+returns it. The first argument, (C<$table>), is an 
+C<SQL::Functional::TableClause> object, which is the table being joined. The 
+second argument is the field on the main table that will be checked. The third 
+argument is the field on the joined table.
+
+=head3 JOIN
+
+Alias for C<INNER_JOIN>.
+
+=head3 LEFT_JOIN
+
+  LEFT_JOIN( $table, $field1, $field2 );
+
+Creates a L<SQL::Functional::JoinClause> with C<join_type = 'left'> and 
+returns it. The first argument, (C<$table>), is an 
+C<SQL::Functional::TableClause> object, which is the table being joined. The 
+second argument is the field on the main table that will be checked. The third 
+argument is the field on the joined table.
+
+=head3 LEFT_OUTER_JOIN
+
+Alias for C<LEFT_JOIN>.
+
+=head3 RIGHT_JOIN
+
+  RIGHT_JOIN( $table, $field1, $field2 );
+
+Creates a L<SQL::Functional::JoinClause> with C<join_type = 'right'> and 
+returns it. The first argument, (C<$table>), is an 
+C<SQL::Functional::TableClause> object, which is the table being joined. The 
+second argument is the field on the main table that will be checked. The third 
+argument is the field on the joined table.
+
+=head3 RIGHT_OUTER_JOIN
+
+Alias for C<RIGHT_JOIN>.
+
+=head3 FULL_JOIN
+
+  FULL_JOIN( $table, $field1, $field2 );
+
+Creates a L<SQL::Functional::JoinClause> with C<join_type = 'full'> and 
+returns it. The first argument, (C<$table>), is an 
+C<SQL::Functional::TableClause> object, which is the table being joined. The 
+second argument is the field on the main table that will be checked. The third 
+argument is the field on the joined table.
+
+=head3 FULL_OUTER_JOIN
+
+Alias for C<FULL_JOIN>.
 
 =head3 SUBSELECT
 
@@ -666,6 +856,34 @@ check as being null.
 Creates a L<SQL::Functional::NullClause> and returns it. Takes a field to 
 check as being not null.
 
+=head3 LIMIT
+
+Creates a L<SQL::Functional::LimitClause> and returns it.  Takes a number 
+that limits the number of rows returned.
+
+=head3 TRUNCATE
+
+Creates a L<SQL::Functional::TruncateClause> and returns it.  Takes a table 
+name.
+
+=head3 DISTINCT
+
+Creates a L<SQL::Functional::DistinictClause> and returns it.  Takes a list of 
+fields, which will be listed as C<DISTINCT> rows in the SQL.
+
+=head3 func
+
+Creates a L<SQL::Functional::FunctionClause> and returns it.  Takes a 
+function name, followed by any parameters.
+
+=head3 literal
+
+Not yet implemented.
+
+When it is, it will create a L<SQL::Functional::LiteralClause> and returns it. 
+Takes a string which will be put literally into the final SQL, rather than as a 
+placeholder.
+
 =head1 WRITING EXTENSIONS
 
 C<SQL::Functional> can be easily extended for new SQL clauses using the
